/*****************************************************************************/
/*                                                                           */
/*                                 ICFILE.H                                  */
/*                                                                           */
/* (C) 1996     Ullrich von Bassewitz                                        */
/*              Wacholderweg 14                                              */
/*              D-70597 Stuttgart                                            */
/* EMail:       uz@ibb.schwaben.com                                          */
/*                                                                           */
/*****************************************************************************/



// $Id$
//
// $Log$
//
//



// Load/save configuration data to a file



#include <stdio.h>

#include "syserror.h"

#include "icfile.h"



/*****************************************************************************/
/*                                   Code                                    */
/*****************************************************************************/



String IstecSaveFile (const String& Filename,
                      const IstecBaseConfig& BaseConfig,
                      const DevConfig* DevConfig)
// Save a configuration to a file. The function returns an error message or
// the empty string if all is well.
{
    unsigned char Buf [BaseConfigSize + IstecDevCount * DevConfigSize];
    unsigned char* B = Buf;

    // Get the firmware version from the base config
    double FirmwareVersion = BaseConfig.GetFirmwareVersion ();

    // Pack the current configuration into the buffer
    BaseConfig.Pack (B);
    B += BaseConfigSize;
    for (unsigned I = 0; I < IstecDevCount; I++) {
        DevConfig [I].Pack (B, FirmwareVersion);
        B += DevConfigSize;
    }

    // Open the file
    FILE* F = fopen (Filename.GetStr (), "wb");
    if (F == NULL) {
        return GetSysErrorMsg (errno);
    }

    // Write the configuration
    fwrite (Buf, sizeof (Buf), 1, F);

    // Close the file
    fclose (F);

    // Success
    return "";
}



String IstecLoadFile (const String& Filename,
                      IstecBaseConfig& BaseConfig,
                      DevConfig* DevConfig)
// Load a configuration from a file. The function returns an error message or
// the empty string if all is well.
{
    // Open the file
    FILE* F = fopen (Filename.GetStr (), "rb");
    if (F == NULL) {
        return GetSysErrorMsg (errno);
    }

    // Read the configuration.
    unsigned char Buf [BaseConfigSize + IstecDevCount * DevConfigSize];
    fread (Buf, sizeof (Buf), 1, F);

    // Close the file
    fclose (F);

    // Unpack the base configuration
    unsigned char* B = Buf;
    BaseConfig.Unpack (B);
    B += BaseConfigSize;

    // Get the firmware version from the base config
    double FirmwareVersion = BaseConfig.GetFirmwareVersion ();

    // Unpack the device config
    for (unsigned I = 0; I < IstecDevCount; I++) {
        DevConfig [I].Unpack (B, FirmwareVersion);
        B += DevConfigSize;
    }

    // Success
    return "";
}


